/*
  swu-unlock: simple Open Lock Daemon client.

  Copyright (c) 2006 Carlo Strozzi

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 dated June, 1991.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Note:

  this program links libraries distributed with the Open Lock Daemon,
  written by Alberto Bertogli (albertogli@telpin.com.ar). According
  to the doc/license file distributed with that program:

    'The act of linking/importing the library into another program
     does NOT in itself make that program considered a derivative
     work of this Original Work.' 

  According to that, it sounds ok if I distribute swu-lock.c under
  the GNU GPL as opposed to the Open Software License that covers
  the Open Lock Daemon code.

*/

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <string.h>

#include "compiler.h"
#include "old.h"

static const char *progname = "swu-unlock";  /* global pointer to argv[0] */

static int eusage(void) {
       fprintf(stderr,\
	 "Usage: %s [-v|--verbose] host resource [resource ...]\n",\
	 progname);
       return 2;
}

int main(int argc, char **argv) {

    int rv=0, i=0, fd, verbose=0, min_args=3;
    char obj[1024];

    char salt[32] = "SWU";			/* default salt value */

    /* the lock password is passed through env, not to
       expose it as a command-line argument. */

    if (getenv("SWU_LOCK_SALT") != NULL)
	strncpy(salt,getenv("SWU_LOCK_SALT"),31);

    while (++i < argc && *argv[i] == '-') {

       min_args++;

       if (!strcmp(argv[i], "-v") ||
	   !strcmp(argv[i], "--verbose")) verbose = 1;
    }

    if (argc < min_args) exit(eusage());

    argv += (min_args - 2);
    argc -= (min_args - 2);

    /* now argv[0] is the hostname */

    fd = old_connect(argv[0], 2626);
    if (fd < 0) {
       fprintf(stderr,\
	   "%s: could not connect to daemon on '%s'\n",progname,argv[0]);
       return 1;
    }

    argv += 1;
    argc -= 1;

    for (i = 0; i < argc; i++) {
        snprintf(obj, 1023, "%s:%s", salt, argv[i]);

	if (verbose) fprintf(stderr,\
	   "%s: trying to unlock resource '%s' ... ",progname,argv[i]);

        rv = old_unlock(fd, obj);

        if (unlikely(rv < 0)) {
	   if (verbose) fprintf(stderr,"failed!\n");
	   exit(1);
	}

	if (verbose) {
	   if (rv) fprintf(stderr,"ok.\n");
	   else fprintf(stderr,"it wasn't locked!\n");
	}
    }

    exit(0);
}

/* EOF */
